'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.parse = exports.decode = offeringTable;

// tuples for the tuple gods!
exports.offeringTable = offeringTable;

exports.stringify = exports.encode = encode;

var safe = require('ini').safe;
var unsafe = require('ini').unsafe;
exports.safe = safe;
exports.unsafe = unsafe;

var eol = require('os').EOL;

function getKey(o) {
  return Object.keys(o)[0];
}

function encode(arr, opt) {
  var children = [];
  var out = '';

  if (typeof opt === 'string') {
    opt = {
      section: opt,
      whitespace: false
    };
  } else {
    opt = opt || {};
    opt.whitespace = opt.whitespace === true;
  }

  var separator = opt.whitespace ? ' = ' : '=';

  arr.forEach(function (el) {

    var k = getKey(el);
    var val = el[k];

    if (Array.isArray(val)) {
      if (_typeof(val[0]) === 'object') {
        // we found a tuple, tuples for the tuple gods!
        children.push(_defineProperty({}, k, val));
      } else {
        // array notation
        val.forEach(function (item) {
          out += safe(k + '[]') + separator + safe(item) + '\n';
        });
      }
    } else {
      out += safe(k) + separator + safe(val) + eol;
    }
  });

  if (opt.section && out.length) {
    out = '[' + safe(opt.section) + ']' + eol + out;
  }

  children.forEach(function (el) {
    var k = getKey(el);

    var nk = dotSplit(k).join('\\.');
    var section = (opt.section ? opt.section + '.' : '') + nk;

    var child = encode(el[k], {
      section: section,
      whitespace: opt.whitespace
    });
    if (out.length && child.length) {
      out += eol;
    }

    out += child;
  });

  return out;
}

function offeringTable(str) {
  var out = [];
  var p = out;
  var section = null;
  //          section     |key      = value
  var re = /^\[([^\]]*)\]$|^([^=]+)(=(.*))?$/i;
  var lines = str.split(/[\r\n]+/g);

  lines.forEach(function (line, _, __) {
    if (!line || line.match(/^\s*[;#]/)) return;
    var match = line.match(re);
    if (!match) return;
    if (match[1] !== undefined) {
      section = unsafe(match[1]);
      p = [];
      out.push(_defineProperty({}, section, p));
      return;
    }
    var key = unsafe(match[2]);
    var value = match[3] ? unsafe(match[4] || '') : true;
    switch (value) {
      case 'true':
      case 'false':
      case 'null':
        value = JSON.parse(value);
    }
    // Convert keys with '[]' suffix to an array
    if (key.length > 2 && key.slice(-2) === '[]') {
      key = key.substring(0, key.length - 2);

      var hasTuple = p.some(function (el) {
        return getKey(el) === key;
      });

      if (!hasTuple) {
        p.push(_defineProperty({}, key, [value]));
        return;
      } else {
        p.forEach(function (el, i) {
          if (getKey(el) === key) {
            var a = el[key];
            a.push(value);
            p[i][key] = a;
          }
        });

        return;
      }
    }

    // safeguard against resetting a previously defined
    // array by accidentally forgetting the brackets
    var e = p[p.length - 1];
    if (e && Array.isArray(e[key])) {
      e[key].push(value);
      return;
    }

    p.push(_defineProperty({}, key, value));
  });

  return out;
}

function dotSplit(str) {
  return str.replace(/\1/g, '\u0002LITERAL\\1LITERAL\u0002').replace(/\\\./g, '\u0001').split(/\./).map(function (part) {
    return part.replace(/\1/g, '\\.').replace(/\2LITERAL\\1LITERAL\2/g, '\u0001');
  });
}
